@@ -2,11 +2,11 @@
 
 import inspect
 import io
-import sys
 from collections import defaultdict
+from collections.abc import Iterable
 from gettext import gettext as _
 from os import getenv
-from typing import Any, DefaultDict, Dict, Iterable, List, Optional, Union
+from typing import Any, Literal, Optional, Union
 
 import click
 from rich import box
@@ -16,16 +16,14 @@ from rich.console import Console, RenderableType, group
 from rich.emoji import Emoji
 from rich.highlighter import RegexHighlighter
 from rich.markdown import Markdown
+from rich.markup import escape
 from rich.padding import Padding
 from rich.panel import Panel
 from rich.table import Table
 from rich.text import Text
 from rich.theme import Theme
-
-if sys.version_info >= (3, 9):
-    from typing import Literal
-else:
-    from typing_extensions import Literal
+from rich.traceback import Traceback
+from typer.models import DeveloperExceptionConfig
 
 # Default styles
 STYLE_OPTION = "bold cyan"
@@ -100,7 +98,7 @@ MARKUP_MODE_MARKDOWN = "markdown"
 MARKUP_MODE_RICH = "rich"
 _RICH_HELP_PANEL_NAME = "rich_help_panel"
 
-MarkupMode = Literal["markdown", "rich", None]
+MarkupModeStrict = Literal["markdown", "rich"]
 
 
 # Rich regex highlighter
@@ -148,11 +146,10 @@ def _get_rich_console(stderr: bool = False) -> Console:
 
 
 def _make_rich_text(
-    *, text: str, style: str = "", markup_mode: MarkupMode
+    *, text: str, style: str = "", markup_mode: MarkupModeStrict
 ) -> Union[Markdown, Text]:
     """Take a string, remove indentations, and return styled text.
 
-    By default, the text is not parsed for any special formatting.
     If `markup_mode` is `"rich"`, the text is parsed for Rich markup strings.
     If `markup_mode` is `"markdown"`, parse as Markdown.
     """
@@ -161,17 +158,16 @@ def _make_rich_text(
     if markup_mode == MARKUP_MODE_MARKDOWN:
         text = Emoji.replace(text)
         return Markdown(text, style=style)
-    if markup_mode == MARKUP_MODE_RICH:
-        return highlighter(Text.from_markup(text, style=style))
     else:
-        return highlighter(Text(text, style=style))
+        assert markup_mode == MARKUP_MODE_RICH
+        return highlighter(Text.from_markup(text, style=style))
 
 
 @group()
 def _get_help_text(
     *,
     obj: Union[click.Command, click.Group],
-    markup_mode: MarkupMode,
+    markup_mode: MarkupModeStrict,
 ) -> Iterable[Union[Markdown, Text]]:
     """Build primary help text for a click command or group.
 
@@ -190,7 +186,8 @@ def _get_help_text(
     help_text = help_text.partition("\f")[0]
 
     # Get the first paragraph
-    first_line = help_text.split("\n\n")[0]
+    first_line, *remaining_paragraphs = help_text.split("\n\n")
+
     # Remove single linebreaks
     if markup_mode != MARKUP_MODE_MARKDOWN and not first_line.startswith("\b"):
         first_line = first_line.replace("\n", " ")
@@ -201,21 +198,11 @@ def _get_help_text(
     )
 
     # Get remaining lines, remove single line breaks and format as dim
-    remaining_paragraphs = help_text.split("\n\n")[1:]
     if remaining_paragraphs:
-        if markup_mode != MARKUP_MODE_RICH:
-            # Remove single linebreaks
-            remaining_paragraphs = [
-                x.replace("\n", " ").strip()
-                if not x.startswith("\b")
-                else "{}\n".format(x.strip("\b\n"))
-                for x in remaining_paragraphs
-            ]
-            # Join back together
-            remaining_lines = "\n".join(remaining_paragraphs)
-        else:
-            # Join with double linebreaks if markdown
-            remaining_lines = "\n\n".join(remaining_paragraphs)
+        # Add a newline inbetween the header and the remaining paragraphs
+        yield Text("")
+        # Join with double linebreaks for markdown and Rich markup
+        remaining_lines = "\n\n".join(remaining_paragraphs)
 
         yield _make_rich_text(
             text=remaining_lines,
@@ -228,7 +215,7 @@ def _get_parameter_help(
     *,
     param: Union[click.Option, click.Argument, click.Parameter],
     ctx: click.Context,
-    markup_mode: MarkupMode,
+    markup_mode: MarkupModeStrict,
 ) -> Columns:
     """Build primary help text for a click option or argument.
 
@@ -240,7 +227,7 @@ def _get_parameter_help(
     # import here to avoid cyclic imports
     from .core import TyperArgument, TyperOption
 
-    items: List[Union[Text, Markdown]] = []
+    items: list[Union[Text, Markdown]] = []
 
     # Get the environment variable first
 
@@ -286,9 +273,11 @@ def _get_parameter_help(
     # Default value
     # This uses Typer's specific param._get_default_string
     if isinstance(param, (TyperOption, TyperArgument)):
-        if param.show_default:
-            show_default_is_str = isinstance(param.show_default, str)
-            default_value = param._extract_default_help_str(ctx=ctx)
+        default_value = param._extract_default_help_str(ctx=ctx)
+        show_default_is_str = isinstance(param.show_default, str)
+        if show_default_is_str or (
+            default_value is not None and (param.show_default or ctx.show_default)
+        ):
             default_str = param._get_default_string(
                 ctx=ctx,
                 show_default_is_str=show_default_is_str,
@@ -314,7 +303,7 @@ def _get_parameter_help(
 def _make_command_help(
     *,
     help_text: str,
-    markup_mode: MarkupMode,
+    markup_mode: MarkupModeStrict,
 ) -> Union[Text, Markdown]:
     """Build cli help text for a click group command.
 
@@ -341,13 +330,13 @@ def _make_command_help(
 def _print_options_panel(
     *,
     name: str,
-    params: Union[List[click.Option], List[click.Argument]],
+    params: Union[list[click.Option], list[click.Argument]],
     ctx: click.Context,
-    markup_mode: MarkupMode,
+    markup_mode: MarkupModeStrict,
     console: Console,
 ) -> None:
-    options_rows: List[List[RenderableType]] = []
-    required_rows: List[Union[str, Text]] = []
+    options_rows: list[list[RenderableType]] = []
+    required_rows: list[Union[str, Text]] = []
     for param in params:
         # Short and long form
         opt_long_strs = []
@@ -367,7 +356,13 @@ def _print_options_panel(
 
         # Column for a metavar, if we have one
         metavar = Text(style=STYLE_METAVAR, overflow="fold")
-        metavar_str = param.make_metavar()
+        # TODO: when deprecating Click < 8.2, make ctx required
+        signature = inspect.signature(param.make_metavar)
+        if "ctx" in signature.parameters:
+            metavar_str = param.make_metavar(ctx=ctx)
+        else:
+            # Click < 8.2
+            metavar_str = param.make_metavar()  # type: ignore[call-arg]
 
         # Do it ourselves if this is a positional argument
         if (
@@ -423,14 +418,14 @@ def _print_options_panel(
                 ),
             ]
         )
-    rows_with_required: List[List[RenderableType]] = []
+    rows_with_required: list[list[RenderableType]] = []
     if any(required_rows):
         for required, row in zip(required_rows, options_rows):
             rows_with_required.append([required, *row])
     else:
         rows_with_required = options_rows
     if options_rows:
-        t_styles: Dict[str, Any] = {
+        t_styles: dict[str, Any] = {
             "show_lines": STYLE_OPTIONS_TABLE_SHOW_LINES,
             "leading": STYLE_OPTIONS_TABLE_LEADING,
             "box": STYLE_OPTIONS_TABLE_BOX,
@@ -463,12 +458,12 @@ def _print_options_panel(
 def _print_commands_panel(
     *,
     name: str,
-    commands: List[click.Command],
-    markup_mode: MarkupMode,
+    commands: list[click.Command],
+    markup_mode: MarkupModeStrict,
     console: Console,
     cmd_len: int,
 ) -> None:
-    t_styles: Dict[str, Any] = {
+    t_styles: dict[str, Any] = {
         "show_lines": STYLE_COMMANDS_TABLE_SHOW_LINES,
         "leading": STYLE_COMMANDS_TABLE_LEADING,
         "box": STYLE_COMMANDS_TABLE_BOX,
@@ -498,8 +493,8 @@ def _print_commands_panel(
     # available instead of allowing the command column to grow and misalign with
     # other panels.
     commands_table.add_column("Description", justify="left", no_wrap=False, ratio=10)
-    rows: List[List[Union[RenderableType, None]]] = []
-    deprecated_rows: List[Union[RenderableType, None]] = []
+    rows: list[list[Union[RenderableType, None]]] = []
+    deprecated_rows: list[Union[RenderableType, None]] = []
     for command in commands:
         helptext = command.short_help or command.help or ""
         command_name = command.name or ""
@@ -540,7 +535,7 @@ def rich_format_help(
     *,
     obj: Union[click.Command, click.Group],
     ctx: click.Context,
-    markup_mode: MarkupMode,
+    markup_mode: MarkupModeStrict,
 ) -> None:
     """Print nicely formatted help text using rich.
 
@@ -572,8 +567,8 @@ def rich_format_help(
                 (0, 1, 1, 1),
             )
         )
-    panel_to_arguments: DefaultDict[str, List[click.Argument]] = defaultdict(list)
-    panel_to_options: DefaultDict[str, List[click.Option]] = defaultdict(list)
+    panel_to_arguments: defaultdict[str, list[click.Argument]] = defaultdict(list)
+    panel_to_options: defaultdict[str, list[click.Option]] = defaultdict(list)
     for param in obj.get_params(ctx):
         # Skip if option is hidden
         if getattr(param, "hidden", False):
@@ -628,7 +623,7 @@ def rich_format_help(
         )
 
     if isinstance(obj, click.Group):
-        panel_to_commands: DefaultDict[str, List[click.Command]] = defaultdict(list)
+        panel_to_commands: defaultdict[str, list[click.Command]] = defaultdict(list)
         for command_name in obj.list_commands(ctx):
             command = obj.get_command(ctx, command_name)
             if command and not command.hidden:
@@ -684,6 +679,10 @@ def rich_format_error(self: click.ClickException) -> None:
     Called by custom exception handler to print richly formatted click errors.
     Mimics original click.ClickException.echo() function but with rich formatting.
     """
+    # Don't do anything when it's a NoArgsIsHelpError (without importing it, cf. #1278)
+    if self.__class__.__name__ == "NoArgsIsHelpError":
+        return
+
     console = _get_rich_console(stderr=True)
     ctx: Union[click.Context, None] = getattr(self, "ctx", None)
     if ctx is not None:
@@ -713,6 +712,11 @@ def rich_abort_error() -> None:
     console.print(ABORTED_TEXT, style=STYLE_ABORTED)
 
 
+def escape_before_html_export(input_text: str) -> str:
+    """Ensure that the input string can be used for HTML export."""
+    return escape(input_text).strip()
+
+
 def rich_to_html(input_text: str) -> str:
     """Print the HTML version of a rich-formatted input string.
 
@@ -730,3 +734,19 @@ def rich_render_text(text: str) -> str:
     """Remove rich tags and render a pure text representation"""
     console = _get_rich_console()
     return "".join(segment.text for segment in console.render(text)).rstrip("\n")
+
+
+def get_traceback(
+    exc: BaseException,
+    exception_config: DeveloperExceptionConfig,
+    internal_dir_names: list[str],
+) -> Traceback:
+    rich_tb = Traceback.from_exception(
+        type(exc),
+        exc,
+        exc.__traceback__,
+        show_locals=exception_config.pretty_exceptions_show_locals,
+        suppress=internal_dir_names,
+        width=MAX_WIDTH,
+    )
+    return rich_tb
